#!/bin/sh
# Author    : Michele La Monaca
# Mail      : perituro-iliad@yahoo.it
# Date      : 2008-08-25
# Rev       : 0.1
#
# Copyright 2008 Michele La Monaca
# Based on wireless.sh authored by Alexis Riksman, Matthijs van de Water, Edwin Mons
#
# This shell script will start/stop an adhoc connection

cd "`/usr/bin/dirname \"$0\"`"

ethIf=wlan0
ethDrv=cf8385
ethDrv2=cfio

address=192.168.169.1
netmask=255.255.255.0
broadcast=192.168.169.255

OK=0
USAGE_ERR=1
CONF_ERR=2
DHCP_ERR=3
UNKNOWN_ERR=4

adhocUsage()
{
    echo "Usage:"
    echo "   $0 stop"
    echo "   $0 start <ssid> [wep <wepkey>]"
    echo "Note: iLiad will act as a dhcp server"
    exit ${USAGE_ERR}
}

stopDHCPclient()
{
    if [ -e /var/run/udhcpc.$ethIf.pid ]
    then
        count=0
        pid=`cat /var/run/udhcpc.$ethIf.pid`
        while kill -0 ${pid} >/dev/null
        do
            if [ ${count} -gt 5 ]
            then
                kill -9 ${pid}
            else
                sleep 1
                count=$((count+1))
            fi
        done
        rm -f /var/run/udhcpc.$ethIf.pid
    fi
}

stopDHCPserver()
{
    if [ -e /var/run/udhcpd.pid ]
    then
        count=0
        pid=`cat /var/run/udhcpd.pid`
        kill ${pid}
        while kill -0 ${pid} >/dev/null
        do
            if [ ${count} -gt 5 ]
            then
                kill -9 ${pid}
            else
                sleep 1
                count=$((count+1))
            fi
        done
        rm -f /var/run/udhcpd.pid
    fi
}

ifDown()
{
    iwconfig ${ethIf} mode managed
    route delete 0.0.0.0 gw 192.168.169.2 metric 100
    ip addr delete $address broadcast $broadcast dev $ethIf
    ifconfig ${ethIf} down >/dev/null
    iwpriv ${ethIf} deepsleep 1

    echo "-- Interface $ethIf down"
}

wirelessConnect()
{
    # Connect to wireless network
    iwconfig $ethIf mode ad-hoc
    iwconfig $ethIf essid "$ssid"
    rc=$?

    # try again once if essid not found (workaround)
    if [ $rc -eq 251 ]
    then
        iwconfig $ethIf essid "$ssid" mode ad-hoc
        rc=$?
    fi
    if [ $rc -ne 0 ]
    then
        ifDown
        echo "An unkwon error occurred while configuring the wireless interface. Quitting"
        exit ${UNKNOWN_ERR}
    fi
}

########
# MAIN #
########

# Take action
case "$1"
in
    "start")
        # Validate stuff
        ssid="$2"
        enc="$3"
        key="$4"
        [ -n "${ssid}" ] || adhocUsage
        [ -z "${enc}" -o -n "${key}" ] || adhocUsage

        # Bring interface down in case it is (partly) active
        stopDHCPclient
        stopDHCPserver

        # Wake-up wireless controller, but keep interface disabled
        rmmod ${ethDrv} 2>/dev/null
        rmmod ${ethDrv2} 2>/dev/null
        modprobe $ethDrv 2>/dev/null
        
        # Select which encryption mode (NONE/WEP)
        case "${enc}"
        in
            "wep")
                # WEP
                iwconfig $ethIf key s:$key
                iwconfig $ethIf key on
                wirelessConnect    
                ;;
            
            "")
                # No encryption
                echo "No encryption used"
                iwconfig $ethIf key off
                wirelessConnect
                ;;

            *)
                # Unknown mode
                adhocUsage
                ;;
        esac
                    
	# Configure address
	ifconfig $ethIf up address $address netmask $netmask broadcast $broadcast
	route add 0.0.0.0 gw 192.168.169.2 metric 100

        # Start DHCP server
        if ./udhcpd udhcpd.conf
        then
            exit ${OK}
        else
            ifDown
            exit ${DHCP_ERR}
        fi
        ;;

    "stop")
        # Bring interface down
        stopDHCPserver
        ifDown
        exit ${OK}
        ;;

    *)
        adhocUsage
        ;;
esac

exit ${UNKNOWN_ERR}  # code not reached

# vi:set ts=4 sw=4 ai et:
